package com.poststats.rs.api.model;

import java.time.LocalDate;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.poststats.rs.api.Constants;
import com.poststats.rs.api.annotation.GeocodeTarget;
import com.poststats.rs.api.annotation.GeocodeTarget.GeocodeField;
import com.poststats.rs.api.annotation.MapCondition;
import com.poststats.rs.api.annotation.MapEntry;

/**
 * @author brian.long@poststats.com
 */
public abstract class BasePerson<ConcreteT extends BasePerson<ConcreteT>> extends BaseModel<ConcreteT> {

	@JsonProperty
	@MapEntry("prefix")
	private String salutation;

	@JsonProperty(required = true)
	@MapEntry("fname")
	private String firstName;

	@JsonProperty(required = true)
	@MapEntry("lname")
	private String lastName;

	@JsonProperty
	@MapEntry("suffix")
	private String suffix;

	@JsonProperty
	@GeocodeTarget(GeocodeField.FullPostalAddress)
	private String postalAddress;

	@JsonProperty
	@MapEntry("email")
	@MapCondition(rolesAllowed = {
			Constants.ADMIN_ROLE, Constants.MANAGER_ROLE, "event:admin", "event:finance", "event:membership"
	})
	@MapCondition(rolesAllowed = {
			"buddy"
	}, method = "doShowEmailAddress")
	private String emailAddress;

	@JsonProperty
	@MapEntry("cellphone")
	@MapCondition(rolesAllowed = {
			Constants.ADMIN_ROLE, Constants.MANAGER_ROLE, "event:admin", "event:finance", "event:membership"
	})
	@MapCondition(rolesAllowed = {
			"buddy"
	}, method = "doShowPhoneNumber")
	private String mobilePhoneNumber;

	@JsonProperty
	@MapEntry("birth")
	@MapCondition(rolesAllowed = {
			Constants.ADMIN_ROLE, Constants.MANAGER_ROLE
	})
	@MapCondition(rolesAllowed = {
			"buddy"
	}, method = "doShowBirthYear")
	private LocalDate birth;

	@JsonProperty
	@MapEntry("gender")
	private Character gender;

	@JsonProperty
	@MapEntry("height")
	private Short height;

	@JsonProperty
	@MapEntry("weight")
	private Integer weight;

	@JsonProperty
	@MapEntry("occupation")
	private String occupation;

	@JsonProperty
	@MapEntry("healthSetback")
	@MapCondition(rolesAllowed = {
			Constants.ADMIN_ROLE, Constants.MANAGER_ROLE
	})
	private LocalDate healthSetback;

	public String getSalutation() {
		return salutation;
	}

	public void setSalutation(String salutation) {
		this.salutation = salutation;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getSuffix() {
		return suffix;
	}

	public void setSuffix(String suffix) {
		this.suffix = suffix;
	}

	public String getPostalAddress() {
		return postalAddress;
	}

	public void setPostalAddress(String postalAddress) {
		this.postalAddress = postalAddress;
	}

	public String getEmailAddress() {
		return emailAddress;
	}

	public void setEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
	}

	public String getMobilePhoneNumber() {
		return mobilePhoneNumber;
	}

	public void setMobilePhoneNumber(String mobilePhoneNumber) {
		this.mobilePhoneNumber = mobilePhoneNumber;
	}

	public LocalDate getBirth() {
		return birth;
	}

	public void setBirth(LocalDate birth) {
		this.birth = birth;
	}

	public Character getGender() {
		return gender;
	}

	public void setGender(Character gender) {
		this.gender = gender;
	}

	public Short getHeight() {
		return height;
	}

	public void setHeight(Short height) {
		this.height = height;
	}

	public Integer getWeight() {
		return weight;
	}

	public void setWeight(Integer weight) {
		this.weight = weight;
	}

	public String getOccupation() {
		return occupation;
	}

	public void setOccupation(String occupation) {
		this.occupation = occupation;
	}

	public LocalDate getHealthSetback() {
		return healthSetback;
	}

	public void setHealthSetback(LocalDate healthSetback) {
		this.healthSetback = healthSetback;
	}

	public ConcreteT withSalutation(String salutation) {
		this.salutation = salutation;
		return this.withThis();
	}

	public ConcreteT withFirstName(String firstName) {
		this.firstName = firstName;
		return this.withThis();
	}

	public ConcreteT withLastName(String lastName) {
		this.lastName = lastName;
		return this.withThis();
	}

	public ConcreteT withSuffix(String suffix) {
		this.suffix = suffix;
		return this.withThis();
	}

	public ConcreteT withPostalAddress(String postalAddress) {
		this.postalAddress = postalAddress;
		return this.withThis();
	}

	public ConcreteT withEmailAddress(String emailAddress) {
		this.emailAddress = emailAddress;
		return this.withThis();
	}

	public ConcreteT withMobilePhoneNumber(String mobilePhoneNumber) {
		this.mobilePhoneNumber = mobilePhoneNumber;
		return this.withThis();
	}

	public ConcreteT withBirth(LocalDate birth) {
		this.birth = birth;
		return this.withThis();
	}

	public ConcreteT withGender(Character gender) {
		this.gender = gender;
		return this.withThis();
	}

	public ConcreteT withHeight(Short height) {
		this.height = height;
		return this.withThis();
	}

	public ConcreteT withWeight(Integer weight) {
		this.weight = weight;
		return this.withThis();
	}

	public ConcreteT withOccupation(String occupation) {
		this.occupation = occupation;
		return this.withThis();
	}

	public ConcreteT withHealthSetback(LocalDate healthSetback) {
		this.healthSetback = healthSetback;
		return this.withThis();
	}

	boolean doShowEmailAddress() {
		return true;
	}

	boolean doShowPostalAddress() {
		return true;
	}

	boolean doShowPhoneNumber() {
		return true;
	}

	boolean doShowBirthYear() {
		return true;
	}

}
