package com.poststats.rs.api.model;

import java.time.MonthDay;
import java.time.OffsetDateTime;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonProperty.Access;
import com.poststats.rs.api.Constants;
import com.poststats.rs.api.annotation.GeocodeSource;
import com.poststats.rs.api.annotation.GeocodeTarget;
import com.poststats.rs.api.annotation.GeocodeTarget.GeocodeField;
import com.poststats.rs.api.annotation.MapCondition;
import com.poststats.rs.api.annotation.MapEntry;

/**
 * @author brian.long@poststats.com
 */
@JsonIgnoreProperties(ignoreUnknown = true)
public class Person extends BasePerson<Person> implements ReferenceablePerson {

	@JsonProperty(required = true, access = Access.READ_ONLY)
	@MapEntry("personID")
	private long id;

	@JsonProperty
	private String fuzzyLocation;

	@MapEntry
	@MapCondition(rolesAllowed = {
			Constants.ADMIN_ROLE, Constants.MANAGER_ROLE, "event:admin", "event:finance", "event:membership"
	})
	@MapCondition(rolesAllowed = {
			"buddy"
	}, method = "doShowPostalAddress")
	@GeocodeTarget(GeocodeField.AddressLines)
	private String addrlines;

	@MapEntry
	@GeocodeTarget(GeocodeField.City)
	private String addrcity;

	@MapEntry
	@GeocodeTarget(GeocodeField.State)
	private String addrstate;

	@MapEntry
	@GeocodeTarget(GeocodeField.Country)
	private String addrcountry;

	@MapEntry
	@MapCondition(rolesAllowed = {
			Constants.ADMIN_ROLE, Constants.MANAGER_ROLE, "event:admin", "event:finance", "event:membership"
	})
	@MapCondition(rolesAllowed = {
			"buddy"
	}, method = "doShowPostalAddress")
	@GeocodeTarget(GeocodeField.PostalCode)
	private String addrzip;

	@MapEntry
	@GeocodeTarget(GeocodeField.FuzzyPostalAddress)
	private String addrfuzzy;

	@MapEntry
	private Boolean hideEmail = true;

	@MapEntry
	private Boolean hidePhone = true;

	@MapEntry
	protected Boolean hideAddress = true;

	@MapEntry
	protected Boolean hideBirthYear = true;

	@JsonProperty(access = Access.READ_ONLY)
	@MapEntry("birth")
	@MapCondition(rolesAllowed = {
			Constants.AUTH_ROLE
	})
	private MonthDay birthDay;

	@JsonProperty(access = Access.READ_ONLY)
	@MapCondition(rolesAllowed = {
			Constants.ADMIN_ROLE, Constants.MANAGER_ROLE
	})
	@MapEntry
	private OffsetDateTime created;

	@Override
	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

	@JsonProperty(access = Access.READ_ONLY)
	public String getFuzzyLocation() {
		return this.addrfuzzy;
	}

	@Override
	@GeocodeSource
	public String getPostalAddress() {
		return super.getPostalAddress();
	}

	public MonthDay getBirthDay() {
		return birthDay;
	}

	public OffsetDateTime getCreated() {
		return created;
	}

	public void setCreated(OffsetDateTime created) {
		this.created = created;
	}

	public Person withId(long id) {
		this.id = id;
		return this;
	}

	public Person withCreated(OffsetDateTime created) {
		this.created = created;
		return this;
	}

	@Override
	boolean doShowPostalAddress() {
		return Boolean.FALSE.equals(this.hideAddress);
	}

	@Override
	boolean doShowEmailAddress() {
		return Boolean.FALSE.equals(this.hideEmail);
	}

	@Override
	boolean doShowPhoneNumber() {
		return Boolean.FALSE.equals(this.hidePhone);
	}

	@Override
	boolean doShowBirthYear() {
		return Boolean.FALSE.equals(this.hideBirthYear);
	}

}
